import { Message, MessageEmbed, User } from "discord.js";
import Bot from "./Bot";
import { Guild } from "../entity/Guild";

export const successEmbed = (title: string) =>
  new MessageEmbed().setTitle(title).setColor("#00AE86");

export const infoEmbed = (title: string) =>
  new MessageEmbed().setTitle(title).setColor("#3498DB");

export const errorEmbed = (title: string) =>
  new MessageEmbed().setTitle(title).setColor("#B93758");

export const decision = (
  message: Message,
  user: User,
  filter: (m: Message) => boolean,
  initial: MessageEmbed,
  accept: (m: Message) => void,
  cancel: () => void,
  time: number = 15000
) => {
  const init = message.channel.send(initial);
  let done = false;
  const collector = message.channel.createMessageCollector(
    (m: Message) => m.author == user && !done,
    { time }
  );

  collector.on("collect", (m: Message) => {
    collector.stop();

    done = true;
    m.delete().catch(() => {});
    if (!filter(m)) return cancel();
    accept(m);
  });
  collector.on("end", async () => {
    (await init).delete();
    !done && cancel();
  });
};

export const statusEmbed = async (message: Message) => {
  const author = message.member;
  const client = message.client;

  const embed = infoEmbed("Contrition")
    .setDescription("A bot designed to give you a second chance.")
    .setThumbnail(client.user.displayAvatarURL())
    .addField("Prefix", Bot.config.bot.prefix, true)
    .addField("Ping", `${Date.now() - message.createdTimestamp}ms`, true)
    .addField("Author", "`elkell#6226`", true)
    .addField("\u200B", "\u200B");

  const guild = await Guild.findOne({ where: { guild: author.guild.id } });
  if (!guild) {
    embed.addField("Configuration", "This guild is not configured yet.");
  } else {
    const channel = author.guild.channels.cache.get(guild.channel);
    const staff = author.guild.roles.cache.get(guild.staffRole);
    const jailed = author.guild.roles.cache.get(guild.jailedRole);

    if (
      author.hasPermission("ADMINISTRATOR") ||
      author.roles.cache.some((role) => role == staff)
    ) {
      embed.addField(
        "Configuration",
        "```fix\n" +
          `Guild Name: ${guild.name}\n` +
          `Logging Channel: #${channel.name}\n` +
          `Staff Role: @${staff.name}\n` +
          `Restricted Role: @${jailed.name}\n` +
          `Default Rejection Expiry: ${guild.defaultExpiry}s\n` +
          "```"
      );
    }
  }

  const uptime = (new Date().getTime() - client.uptime) / 1e3;
  embed.addField("Uptime", `Since <t:${Math.trunc(uptime)}:R>`, true);
  embed.addField("Source", "[GitHub](https://github.com/hhhapz)", true);
  return embed;
};
