import { Command } from "discord-akairo";
import { Message } from "discord.js";
import { paginate, reactions } from "../../config/Constants";
import { Appeal } from "../../entity/Appeal";
import { Guild } from "../../entity/Guild";
import { infoEmbed } from "../Util";

class History extends Command {
  constructor() {
    super("history", {
      category: "Review",
      aliases: ["History"],
      description: "<user> [page]",
      args: [
        {
          id: "user",
          type: "string",
          prompt: {
            start: () =>
              infoEmbed("The ID for the user:").setFooter(
                'You can cancel this at any time by saying "cancel".'
              ),
          },
        },
        {
          id: "pageNum",
          type: "number",
          default: 1,
        },
      ],
    });
  }

  async exec(
    message: Message,
    { user, pageNum }: { user: string; pageNum: number }
  ) {
    const userRe = /<@!?(\d+)>/.exec(user);
    if (userRe) {
      user = userRe[1];
    }

    const guild = await Guild.findOneOrFail({
      where: { guild: message.member.guild.id },
    });
    const appeals = await Appeal.allAppeals(user, guild.id);
    const page = paginate(appeals, 10, pageNum);

    if (appeals.length === 0) {
      message.channel.send(infoEmbed(`The specified user has 0 appeals.`));
      return;
    }

    const start = (pageNum - 1) * 10 + 1;
    const footer = `Showing appeals ${start} to ${
      start + page.length - 1
    }. Total ${appeals.length} appeal(s).`;

    const embed = this.createEmbed(page[0], start, footer);
    const msg = await message.channel.send(embed);

    if (appeals.length === 1) {
      return;
    }

    page.forEach((_, i) => msg.react(reactions[i]).catch(() => {}));

    const collector = msg.createReactionCollector(
      (reaction, user) => {
        return (
          !user.bot &&
          reactions.slice(0, page.length).indexOf(reaction.emoji.name) != -1
        );
      },
      { time: 60000 }
    );
    collector.on("collect", (reaction, user) => {
      const index = appeals.indexOf(
        page[reactions.indexOf(reaction.emoji.name)]
      );
      msg.edit(this.createEmbed(appeals[index], index + 1, footer));
      msg.reactions.resolve(reaction.emoji.name).users.remove(user);
    });
    collector.on("end", () => {
      msg.reactions.removeAll().catch(() => {});
    });
  }

  createEmbed(appeal: Appeal, index: number, footer: string) {
    const embed = appeal.embed(false);
    embed
      .setTitle(`${embed.title} - (${index})`)
      .setFooter(`${embed.footer.text}\n${footer}`);
    return embed;
  }
}

export default History;
