import { Command } from "discord-akairo";
import { Message, TextChannel } from "discord.js";
import { cancelString } from "../../config/Constants";
import { Appeal } from "../../entity/Appeal";
import { errorEmbed, infoEmbed, successEmbed } from "../Util";

class RejectCommand extends Command {
  constructor() {
    super("reject", {
      category: "Review",
      aliases: ["Reject"],
      description: "<appeal> [decision]",
      clientPermissions: ["CREATE_INSTANT_INVITE", "BAN_MEMBERS"],
      channel: "guild",
      args: [
        {
          id: "appeal",
          type: "appeal",
          prompt: {
            start: () => infoEmbed("The appeal ID:").setFooter(cancelString),
          },
        },
        {
          id: "reason",
          type: "string",
          prompt: {
            start: () =>
              infoEmbed("(Optional) Decision note for appeal:")
                .setDescription("Enter `none` to provide no reason")
                .setFooter(cancelString),
          },
        },
      ],
    });
  }

  async exec(message: Message, args: { appeal: Appeal; reason: string }) {
    let { appeal, reason } = args;
    if (reason === "none") reason = null;

    if (["accepted", "rejected"].includes(appeal.status)) {
      return message.channel.send(
        errorEmbed("Could not accept")
          .addField("Error", "Cannot reject appeal with decision already made.")
          .addField("Status", appeal.status)
          .setFooter(appeal.id)
      );
    }

    appeal.reject(message.author.id, reason);

    const channel = await message.client.channels
      .fetch(appeal.guild.channel)
      .catch(() => null);

    if (!channel || !channel?.isText()) {
      return errorEmbed("Could not update. Guild configuration invalid.");
    }

    const origMsg = await (channel as TextChannel).messages
      .fetch(appeal.message)
      .catch(() => null);

    if (!origMsg) {
      return errorEmbed("Could not update original message");
    }

    message.channel.send(
      successEmbed("The appeal has been rejected.")
        .setDescription(`${appeal.userTag} will stay banned.`)
        .addField("Original Message", `[link](${appeal.link()})`)
        .setFooter(appeal.id)
    );
    origMsg.edit(appeal.embed(true).setColor("#B93758"));

    appeal.save();
  }
}

export default RejectCommand;
