import { Command } from "discord-akairo";
import { TextChannel } from "discord.js";
import { Message } from "discord.js";
import { cancelString } from "../../config/Constants";
import { Appeal } from "../../entity/Appeal";
import { errorEmbed, infoEmbed, successEmbed } from "../Util";

class SetExpiry extends Command {
  constructor() {
    super("setExpiry", {
      category: "Review",
      aliases: ["SetExpiry"],
      description: "<appeal> <date>",
      clientPermissions: ["CREATE_INSTANT_INVITE", "BAN_MEMBERS"],
      channel: "guild",
      args: [
        {
          id: "appeal",
          type: "appeal",
          prompt: {
            start: () => infoEmbed("The appeal ID:").setFooter(cancelString),
          },
        },
        {
          id: "date",
          type: "optionalDate",
          prompt: {
            start: () =>
              infoEmbed("The date of expiry:")
                .setDescription(
                  "Specify JS dateString,\nor specify `never`,\nor specify `now`.\n\nDate must be in the future."
                )
                .setFooter(cancelString),
          },
        },
      ],
    });
  }

  async exec(
    message: Message,
    args: { appeal: Appeal; date: Date | "never" | "now" }
  ) {
    const { appeal, date } = args;
    if (appeal.status !== "rejected") {
      return message.reply(
        errorEmbed("Invalid Appeal Status")
          .setDescription("The specified appeal's status must be 'rejected'.")
          .addField("Current Appeal Status", appeal.status)
      );
    }

    if (date === "never") appeal.rejectionExpiry = null;
    else if (date === "now") appeal.rejectionExpiry = new Date();
    else appeal.rejectionExpiry = date;

    if (date < new Date()) {
      return message.reply(
        errorEmbed("Invalid Date provided").addField(
          "Parsed Date:",
          date.toString()
        )
      );
    }

    appeal.save();
    message.reply(
      successEmbed("Appeal expiry updated").setDescription(
        `View original message: [link](${appeal.link()}).`
      )
    );

    const channel = await message.client.channels
      .fetch(appeal.guild.channel)
      .catch(() => null);

    if (!channel || !channel?.isText()) {
      return errorEmbed("Could not update. Guild configuration invalid.");
    }

    const msg = await (channel as TextChannel).messages
      .fetch(appeal.message)
      .catch(() => null);
    if (!msg) {
      return errorEmbed("Could not update original message");
    }
    msg.edit(appeal.embed(true).setColor("#B93758"));
  }
}

export default SetExpiry;
