import { Command } from "discord-akairo";
import { Channel } from "discord.js";
import { Role } from "discord.js";
import { Message } from "discord.js";
import { cancelString } from "../../config/Constants";
import { Guild } from "../../entity/Guild";

import { errorEmbed, infoEmbed, successEmbed } from "../Util";

class Setup extends Command {
  constructor() {
    super("setup", {
      category: "Utility",
      aliases: ["Setup"],
      clientPermissions: ["CREATE_INSTANT_INVITE", "BAN_MEMBERS"],
      userPermissions: ["ADMINISTRATOR"],
      channel: "guild",
      args: [
        {
          id: "name",
          type: "string",
          prompt: {
            start: () =>
              infoEmbed("Please provide the following information.")
                .setDescription("1. Supply a name for this guild.")
                .setFooter(cancelString),
          },
        },
        {
          id: "channel",
          type: "textChannel",
          prompt: {
            start: () =>
              successEmbed("Success")
                .setDescription("2. Supply a channel for posting appeals.")
                .setFooter(cancelString),
            retry: errorEmbed("Error")
              .setDescription(
                "Please try again. Mention a channel or provide its ID."
              )
              .setFooter(cancelString),
          },
        },
        {
          id: "staffRole",
          type: "role",
          prompt: {
            start: () =>
              successEmbed("Success")
                .setDescription(
                  "3. Enter the staff role ID or mention the staff role to access and use this bot"
                )
                .setFooter(cancelString),
            retry: errorEmbed("Error")
              .setDescription(
                "Please try again. Mention a role or provide its ID."
              )
              .setFooter(cancelString),
          },
        },
        {
          id: "jailedRole",
          type: "role",
          prompt: {
            start: () =>
              successEmbed("Success")
                .setDescription(
                  "3. Enter the role ID or mention the role for users who join for further questioning (i.e the muted role)"
                )
                .setFooter(cancelString),
            retry: errorEmbed("Error")
              .setDescription(
                "Please try again. Mention a role or provide its ID."
              )
              .setFooter(cancelString),
          },
        },
        {
          id: "defaultExpiry",
          type: "timestring",
          prompt: {
            start: () =>
              successEmbed("Success")
                .setDescription(
                  "4. Enter a time string for the minimum time before users can appeal after a rejection."
                )
                .addField(
                  "Valid Durations",
                  "```1d - days\n1w - weeks\n1m - months\n1y - years```"
                )
                .addField(
                  "Examples",
                  "```\n1m    - 1 month\n1d    - 1 day\n1m 3d - 1 month 3 days\n1y1m  - 1 year 1 month```"
                )
                .setFooter(cancelString),
            retry: errorEmbed("Error")
              .setDescription("Please try again with a valid time string.")
              .setFooter(cancelString),
          },
        },
      ],
    });
  }

  async exec(
    message: Message,
    args: {
      name: string;
      channel: Channel;
      staffRole: Role;
      jailedRole: Role;
      defaultExpiry: string;
    }
  ) {
    const { name, channel, staffRole, jailedRole, defaultExpiry } = args;
    const author = message.member;
    if (author === null) {
      return;
    }

    if (await Guild.count({ where: { guild: author.guild.id } })) {
      return message.channel.send(errorEmbed("This guild is already setup."));
    }

    if (await Guild.count({ where: { name: args.name } })) {
      return message.channel.send(
        errorEmbed("A guild has already registered with that name.")
      );
    }

    const guild = new Guild(
      name,
      author.guild.id,
      channel.id,
      staffRole.id,
      jailedRole.id,
      defaultExpiry
    );
    guild.save();

    message.channel.send(
      successEmbed("Guild Setup Complete")
        .setDescription(
          "The guild has successfully been setup, and will listen for appeal requests."
        )
        .addField("Name", name, true)
        .addField("Post Channel", `<#${guild.channel}>`, true)
        .addField("\u200b", "\u200b", true)
        .addField("Staff Role", `<@&${guild.staffRole}>`, true)
        .addField("Restricted Role", `<@&${guild.jailedRole}>`, true)
        .addField("\u200b", "\u200b", true)
        .addField("Minimum time before re-appeal", defaultExpiry || "Never")
    );
  }
}

export default Setup;
