export const PORT = process.env.PORT || 4000;

export const OAUTH_SCOPE = "identify guilds.join";

export const API_URL = "https://discord.com/api/v6";

export const CDN_URL = "https://cdn.discordapp.com/";

export const cancelString =
  'You can cancel this at any time by saying "cancel".';

export const reactions = [
  "1️⃣",
  "2️⃣",
  "3️⃣",
  "4️⃣",
  "5️⃣",
  "6️⃣",
  "7️⃣",
  "8️⃣",
  "9️⃣",
  "🔟",
];

export const paginate = <T>(
  array: Array<T>,
  size: number,
  pageNum: number
): Array<T> => {
  return array.slice((pageNum - 1) * size, pageNum * size);
};

export const parseDuration = (timeString: string): string => {
  const secs = timeString.match(/(\d+)\s*s/);
  const days = timeString.match(/(\d+)\s*d/);
  const weeks = timeString.match(/(\d+)\s*w/);
  const months = timeString.match(/(\d+)\s*m/);
  const years = timeString.match(/(\d+)\s*y/);

  let str = "";
  if (secs) str += `${secs[1]}s `;
  if (days) str += `${days[1]}d `;
  if (weeks) str += `${weeks[1]}w `;
  if (months) str += `${months[1]}m `;
  if (years) str += `${years[1]}y `;

  return str.trim();
};

export const getDate = (timeString: string): Date => {
  const now = new Date();
  const date = new Date(now);

  if (timeString === "never") {
    return now;
  }

  const secs = timeString.match(/(\d+)\s*s/);
  const days = timeString.match(/(\d+)\s*d/);
  const weeks = timeString.match(/(\d+)\s*w/);
  const months = timeString.match(/(\d+)\s*m/);
  const years = timeString.match(/(\d+)\s*y/);

  if (secs) date.setSeconds(date.getSeconds() + parseInt(secs[1]));
  if (days) date.setDate(date.getDate() + parseInt(days[1]));
  if (weeks) date.setDate(date.getDate() + parseInt(weeks[1]) * 7);
  if (months) date.setMonth(date.getMonth() + parseInt(months[1]));
  if (years) date.setFullYear(date.getFullYear() + parseInt(years[1]));

  if (now.getTime() === date.getTime()) return null;
  return date;
};
