import { Request, Response } from "express";

/**
 * List of API routes with examples
 * @route GET /api
 */
export const getApi = (_: Request, res: Response) => {
  res.json({
    "POST /oauth": {
      summary: "Create a request to generate an OAuth token URL from Discord.",
      responses: {
        200: {
          description: "The OAuth URL",
          format: {
            redirectURL: {
              type: "string",
              format: "url",
            },
          },
        },
      },
    },
    "GET /authorize": {
      summary: "Callback from discord with auth token",
      parameters: {
        query: {
          code: {
            type: "string",
          },
          state: {
            type: "string",
          },
        },
      },
      responses: {
        200: {
          description: "Successfully complete OAuth exchange",
        },
        400: {
          description: "An invalid code or state parameter was provided",
          format: {
            reason: {
              type: "string",
              format: "error",
            },
          },
        },
        500: {
          description: "Something went wrong on the server",
          format: {
            reason: {
              type: "string",
              format: "error",
              optional: true,
            },
          },
        },
      },
    },
    "POST /submit": {
      summary: "Post an unban request appeal",
    },
  });
};
