import { Request, Response } from "express";
import { Config } from "../config/Config";
import { DiscordToken } from "../entity/DiscordToken";
import { StateToken } from "../entity/StateToken";
import { fail, respond } from "./Util";

export const authorize = async (req: Request, res: Response) => {
  const config: Config = req.app.get("config");
  const { code, state } = req.body;

  // enforce types
  if (!(typeof code === "string" && typeof state === "string")) {
    return fail(res, 500, "Invalid request body");
  }

  const stateToken = await StateToken.findOne({ where: { token: state } });
  if (!stateToken) {
    return fail(res, 400, "Invalid state provided");
  }

  let dToken: DiscordToken;
  try {
    dToken = await stateToken.verify(config, code.toString());
  } catch ({ reason, status }) {
    fail(res, status, reason);
  }

  respond(res, { token: dToken.id });
};
