import { Request, Response } from "express";
import { DiscordCache } from "../entity/DiscordCache";
import { DiscordToken } from "../entity/DiscordToken";
import { Guild } from "../entity/Guild";
import { fail, respond } from "./Util";

export const info = async (req: Request, res: Response) => {
  const { token } = req.body;

  // enforce types
  if (!(typeof token === "string")) {
    return fail(res, 500, "Invalid request body");
  }

  const discordToken = await DiscordToken.findOne({ where: { id: token } });
  if (!discordToken) {
    return fail(res, 400, "Invalid token provided");
  }

  let cache: DiscordCache;
  try {
    cache = await discordToken.queryData();
  } catch ({ status, reason }) {
    fail(res, status, reason);
    return;
  }

  let platforms: Guild[];
  try {
    platforms = await Guild.all();
  } catch (err) {
    console.error("Could not fetch guilds", err);
    return fail(res, 500, "An internal error occurred.");
  }

  respond(res, {
    id: cache.userID,
    username: cache.username,
    discriminator: cache.discriminator,
    avatar: cache.avatar,
    platforms: platforms.map((g) => ({
      name: g.name,
      id: g.id,
    })),
  });
};
