import { Request, Response } from "express";
import { Appeal } from "../entity/Appeal";
import { DiscordCache } from "../entity/DiscordCache";
import { DiscordToken } from "../entity/DiscordToken";
import { fail, respond } from "./Util";

export const status = async (req: Request, res: Response) => {
  const { token } = req.body;

  // enforce types
  if (!(typeof token === "string")) {
    return fail(res, 500, "Invalid request body");
  }

  const discordToken = await DiscordToken.findOne({ where: { id: token } });
  if (!discordToken) {
    res.status(400);
    res.json({ reason: "Invalid token provided", code: 400 });
    return;
  }

  // TODO: check with bot if the user is banned.
  let cache: DiscordCache;
  try {
    cache = await discordToken.queryData();
  } catch ({ status, reason }) {
    return fail(res, status, reason);
  }

  let appeals: Appeal[];
  try {
    appeals = await Appeal.allAppeals(cache.userID);
  } catch (err) {
    console.error("Could not fetch appeals", err);
    return fail(res, 500, "An internal error occurred.");
  }

  respond(
    res,
    appeals.map((appeal) => {
      return {
        ...appeal,
        message: undefined,
        decisionBy: undefined,
        userTag: undefined,
        userAvatar: undefined,
        guild: appeal.guild.name,
      };
    })
  );
};
